#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#
# Copyright (c) 2008-2015 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#

import datetime
import time
import logging

from django.conf import settings
from django import forms
from django.forms.formsets import formset_factory
from django.core.urlresolvers import reverse

from omeroweb.custom_forms import NonASCIIForm
from .custom_forms import MetadataModelChoiceField
from .custom_forms import AnnotationModelMultipleChoiceField
from .custom_forms import ObjectModelMultipleChoiceField
from omeroweb.webadmin.custom_forms import ExperimenterModelMultipleChoiceField
from omeroweb.webadmin.custom_forms import GroupModelMultipleChoiceField
from omeroweb.webadmin.custom_forms import GroupModelChoiceField
from omeroweb.webclient.webclient_utils import formatPercentFraction

logger = logging.getLogger(__name__)


##################################################################
# Static values

# TODO: change to reverse
help_button = "%swebgateway/img/help16.png" % settings.STATIC_URL

help_enable = (
    '<span class="tooltip" title="Enable/Disable: This option'
    ' allows the owner to keep the access control of the share.">'
    '<img src="%s" /></span>') % help_button

help_expire = (
    '<span class="tooltip" title="Expiry date: This date defines'
    ' when the share will stop being available. Date format:'
    ' YYYY-MM-DD."><img src="%s" /></span>') % help_button


#################################################################
# Non-model Form


class GlobalSearchForm(NonASCIIForm):

    search_query = forms.CharField(widget=forms.TextInput(attrs={'size': 25}))


class ShareForm(NonASCIIForm):

    def __init__(self, *args, **kwargs):
        super(ShareForm, self).__init__(*args, **kwargs)

        try:
            if kwargs['initial']['shareMembers']:
                pass
            self.fields['members'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'],
                initial=kwargs['initial']['shareMembers'],
                widget=forms.SelectMultiple(attrs={'size': 28}))
        except Exception:
            self.fields['members'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'],
                widget=forms.SelectMultiple(attrs={'size': 28}))
        self.fields.keyOrder = [
            'message', 'expiration', 'enable', 'members']  # , 'guests']

    message = forms.CharField(
        widget=forms.Textarea(attrs={'rows': 5, 'cols': 50}))
    expiration = forms.CharField(
        max_length=100,
        widget=forms.TextInput(attrs={'size': 10}),
        label="Expiry date",
        help_text=help_expire,
        required=False)
    enable = forms.BooleanField(required=False, help_text=help_enable)
    # guests = MultiEmailField(required=False,
    # widget=forms.TextInput(attrs={'size':75}))

    def clean_expiration(self):
        if (self.cleaned_data['expiration'] is not None and
                len(self.cleaned_data['expiration']) < 1):
            return None
        if self.cleaned_data['expiration'] is not None:
            d = str(self.cleaned_data['expiration']).rsplit("-")
            try:
                date = datetime.datetime.strptime(
                    ("%s-%s-%s" % (d[0], d[1], d[2])), "%Y-%m-%d")
            except Exception:
                raise forms.ValidationError(
                    'Date is in the wrong format. YY-MM-DD')
            if time.mktime(date.timetuple()) <= time.time():
                raise forms.ValidationError(
                    'Expiry date must be in the future.')
        return self.cleaned_data['expiration']


class BasketShareForm(ShareForm):

    def __init__(self, *args, **kwargs):
        super(BasketShareForm, self).__init__(*args, **kwargs)

        try:
            self.fields['image'] = GroupModelMultipleChoiceField(
                queryset=kwargs['initial']['images'],
                initial=kwargs['initial']['selected'],
                widget=forms.SelectMultiple(attrs={'size': 10}))
        except Exception:
            self.fields['image'] = GroupModelMultipleChoiceField(
                queryset=kwargs['initial']['images'],
                widget=forms.SelectMultiple(attrs={'size': 10}))


class ContainerForm(NonASCIIForm):

    name = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 45}))
    description = forms.CharField(
        widget=forms.Textarea(attrs={'rows': 2, 'cols': 49}),
        required=False)
    owner = forms.CharField(
        widget=forms.HiddenInput,
        required=False)


class ContainerNameForm(NonASCIIForm):

    name = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 45}))


class ContainerDescriptionForm(NonASCIIForm):

    description = forms.CharField(
        widget=forms.Textarea(attrs={'rows': 3, 'cols': 39}),
        required=False)


class BaseAnnotationForm(NonASCIIForm):
    """
    This is the superclass of the various forms used for annotating single or
    multiple objects.
    All these forms use hidden fields to specify the object(s) currently being
    annotated.
    """
    def __init__(self, *args, **kwargs):
        super(BaseAnnotationForm, self).__init__(*args, **kwargs)

        images = ('images' in kwargs['initial'] and
                  kwargs['initial']['images'] or list())
        if len(images) > 0:
            try:
                self.fields['image'] = ObjectModelMultipleChoiceField(
                    queryset=images,
                    initial=kwargs['initial']['selected']['images'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['image'] = ObjectModelMultipleChoiceField(
                    queryset=images,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)

        datasets = ('datasets' in kwargs['initial'] and
                    kwargs['initial']['datasets'] or list())
        if len(datasets) > 0:
            try:
                self.fields['dataset'] = ObjectModelMultipleChoiceField(
                    queryset=datasets,
                    initial=kwargs['initial']['selected']['datasets'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['dataset'] = ObjectModelMultipleChoiceField(
                    queryset=datasets,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)

        projects = ('projects' in kwargs['initial'] and
                    kwargs['initial']['projects'] or list())
        if len(projects) > 0:
            try:
                self.fields['project'] = ObjectModelMultipleChoiceField(
                    queryset=projects,
                    initial=kwargs['initial']['selected']['projects'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['project'] = ObjectModelMultipleChoiceField(
                    queryset=projects,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)

        screens = ('screens' in kwargs['initial'] and
                   kwargs['initial']['screens'] or list())
        if len(screens) > 0:
            try:
                self.fields['screen'] = ObjectModelMultipleChoiceField(
                    queryset=screens,
                    initial=kwargs['initial']['selected']['screens'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['screen'] = ObjectModelMultipleChoiceField(
                    queryset=screens,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)

        plates = ('plates' in kwargs['initial'] and
                  kwargs['initial']['plates'] or list())
        if len(plates) > 0:
            try:
                self.fields['plate'] = ObjectModelMultipleChoiceField(
                    queryset=plates,
                    initial=kwargs['initial']['selected']['plates'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['plate'] = ObjectModelMultipleChoiceField(
                    queryset=plates,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)

        acquisitions = ('acquisitions' in kwargs['initial'] and
                        kwargs['initial']['acquisitions'] or list())
        if len(acquisitions) > 0:
            try:
                self.fields['acquisition'] = ObjectModelMultipleChoiceField(
                    queryset=acquisitions,
                    initial=kwargs['initial']['selected']['acquisitions'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['acquisition'] = ObjectModelMultipleChoiceField(
                    queryset=acquisitions,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)

        wells = ('wells' in kwargs['initial'] and
                 kwargs['initial']['wells'] or list())
        if len(wells) > 0:
            try:
                self.fields['well'] = ObjectModelMultipleChoiceField(
                    queryset=wells,
                    initial=kwargs['initial']['selected']['wells'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['well'] = ObjectModelMultipleChoiceField(
                    queryset=wells,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)

        shares = ('shares' in kwargs['initial'] and
                  kwargs['initial']['shares'] or list())
        if len(shares) > 0:
            try:
                self.fields['share'] = ObjectModelMultipleChoiceField(
                    queryset=shares,
                    initial=kwargs['initial']['selected']['shares'],
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)
            except Exception:
                self.fields['share'] = ObjectModelMultipleChoiceField(
                    queryset=shares,
                    widget=forms.SelectMultiple(attrs={'size': 10}),
                    required=False)


class TagsAnnotationForm(BaseAnnotationForm):
    """
    Form for annotating one or more objects with existing Tags or New tags
    """

    def __init__(self, *args, **kwargs):
        super(TagsAnnotationForm, self).__init__(*args, **kwargs)

    tags = forms.CharField(required=False, widget=forms.HiddenInput)

    def clean_tags(self):
        data = self.cleaned_data['tags']
        if not data:
            return []
        try:
            data = map(int, data.split(','))
        except Exception:
            raise forms.ValidationError()
        return data


class NewTagsAnnotationForm(forms.Form):
    """ Helper form for new tags """
    tag = forms.CharField(required=True, widget=forms.HiddenInput)
    description = forms.CharField(required=False, widget=forms.HiddenInput)
    tagset = forms.IntegerField(
        min_value=1, required=False, widget=forms.HiddenInput)


NewTagsAnnotationFormSet = formset_factory(NewTagsAnnotationForm, extra=0)


class FilesAnnotationForm(BaseAnnotationForm):

    def __init__(self, *args, **kwargs):
        super(FilesAnnotationForm, self).__init__(*args, **kwargs)
        self.fields['files'] = AnnotationModelMultipleChoiceField(
            queryset=kwargs['initial']['files'],
            widget=forms.SelectMultiple(attrs={
                'size': 8, 'class': 'existing'}),
            required=False)

    annotation_file = forms.FileField(required=False)


class CommentAnnotationForm(BaseAnnotationForm):
    comment = forms.CharField(
        widget=forms.Textarea(attrs={'rows': 2, 'cols': 39}))


class ActiveGroupForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(ActiveGroupForm, self).__init__(*args, **kwargs)
        try:
            self.fields['active_group'] = GroupModelChoiceField(
                queryset=kwargs['initial']['mygroups'],
                initial=kwargs['initial']['activeGroup'],
                empty_label=None,
                widget=forms.Select(attrs={
                    'onchange': (
                        'window.location.href=\'' +
                        reverse(viewname="change_active_group") +
                        '?url=' + kwargs['initial']['url'] +
                        '&active_group=\''
                        '+this.options[this.selectedIndex].value')}))
        except Exception:
            self.fields['active_group'] = GroupModelChoiceField(
                queryset=kwargs['initial']['mygroups'],
                initial=kwargs['initial']['activeGroup'],
                empty_label=None, widget=forms.Select(attrs={
                    'onchange': (
                        'window.location.href=\'' +
                        reverse(viewname="change_active_group") +
                        '?active_group=\''
                        '+this.options[this.selectedIndex].value')}))
        self.fields.keyOrder = ['active_group']


class WellIndexForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(WellIndexForm, self).__init__(*args, **kwargs)
        rmin, rmax = kwargs['initial']['range']
        choices = [(str(i), "Field#%i" % (i-rmin+1))
                   for i in range(rmin, rmax+1)]
        self.fields['index'] = forms.ChoiceField(
            choices=tuple(choices),
            widget=forms.Select(attrs={
                'onchange': (
                    'changeField(this.options[this.selectedIndex].value);')}))
        self.fields.keyOrder = ['index']

###############################
# METADATA FORMS


def save_metadata(obj, name, options=False):
    s = 'javascript:save_metadata(' + str(obj) + ', \'' + name + '\', '
    if options:
        s += 'this.options[this.selectedIndex].value);'
    else:
        s += 'this.value);'

    return s


def set_widget_attrs(field, set_class=True):
    field.widget.attrs['disabled'] = True
    if set_class:
        field.widget.attrs['class'] = 'disabled-metadata'


class MetadataChannelForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataChannelForm, self).__init__(*args, **kwargs)

        # Logical channel

        # Name
        logicalCh = kwargs['initial']['logicalChannel']
        try:
            if logicalCh is not None:
                self.fields['name'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id)}),
                    initial=logicalCh.name,
                    required=False)
            else:
                self.fields['name'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    required=False)
            set_widget_attrs(self.fields['name'])
        except Exception:
            self.fields['name'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['name'])

        # excitationWave
        try:
            if logicalCh is not None:
                self.fields['excitationWave'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    initial=kwargs['initial']['exWave'].getValue(),
                    label=("Excitation (%s)"
                           % kwargs['initial']['exWave'].getSymbol()),
                    required=False)
            else:
                self.fields['excitationWave'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    label="Excitation",
                    required=False)
            set_widget_attrs(self.fields['excitationWave'])
        except Exception:
            self.fields['excitationWave'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Excitation",
                required=False)
            set_widget_attrs(self.fields['excitationWave'])

        # emissionWave
        try:
            if logicalCh is not None:
                self.fields['emissionWave'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    initial=kwargs['initial']['emWave'].getValue(),
                    label=("Emission (%s)"
                           % kwargs['initial']['emWave'].getSymbol()),
                    required=False)
            else:
                self.fields['emissionWave'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    label="Emission",
                    required=False)
            set_widget_attrs(self.fields['emissionWave'])
        except Exception:
            self.fields['emissionWave'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Emission",
                required=False)
            set_widget_attrs(self.fields['emissionWave'])

        # ndFilter
        try:
            if logicalCh is not None and logicalCh.ndFilter is not None:
                self.fields['ndFilter'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    initial=formatPercentFraction(logicalCh.ndFilter),
                    label="ND filter (%)",
                    required=False)
            else:
                self.fields['ndFilter'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    label="ND filter (%)",
                    required=False)
            set_widget_attrs(self.fields['ndFilter'], set_class=False)
        except Exception:
            self.fields['ndFilter'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="ND filter (%)",
                required=False)
            set_widget_attrs(self.fields['ndFilter'], set_class=False)

        # pinHoleSize
        try:
            if logicalCh is not None and logicalCh.pinHoleSize is not None:
                self.fields['pinHoleSize'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    initial=logicalCh.pinHoleSize.getValue(),
                    label=("Pin hole size (%s)"
                           % logicalCh.pinHoleSize.getSymbol()),
                    required=False)
            else:
                self.fields['pinHoleSize'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    label="Pin hole size",
                    required=False)
            set_widget_attrs(self.fields['pinHoleSize'], set_class=False)
        except Exception:
            self.fields['pinHoleSize'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Pin hole size",
                required=False)
            set_widget_attrs(self.fields['pinHoleSize'], set_class=False)

        # fluor
        try:
            if logicalCh is not None:
                self.fields['fluor'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    initial=logicalCh.fluor,
                    required=False)
            else:
                self.fields['fluor'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    required=False)
            set_widget_attrs(self.fields['fluor'], set_class=False)
        except Exception:
            self.fields['fluor'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['fluor'], set_class=False)

        # Illumination
        try:
            if logicalCh.getIllumination() is not None:
                self.fields['illumination'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['illuminations'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            logicalCh.id, 'illumination', options=True)}),
                    initial=logicalCh.getIllumination(),
                    required=False)
            else:
                self.fields['illumination'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['illuminations'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            logicalCh.id, 'illumination', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['illumination'], set_class=False)
        except Exception:
            self.fields['illumination'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['illumination'], set_class=False)

        # contrastMethods
        try:
            if logicalCh.contrastMethod is not None:
                self.fields['contrastMethod'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['contrastMethods'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            logicalCh.id, 'contrastMethod', options=True)}),
                    initial=logicalCh.getContrastMethod(),
                    label="Contrast method",
                    required=False)
            else:
                self.fields['contrastMethod'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['contrastMethods'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            logicalCh.id, 'contrastMethod', options=True)}),
                    label="Contrast method",
                    required=False)
            set_widget_attrs(self.fields['contrastMethod'])
        except Exception:
            self.fields['contrastMethod'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Contrast method",
                required=False)
            set_widget_attrs(self.fields['contrastMethod'])

        # Mode
        try:
            if logicalCh.getMode() is not None:
                self.fields['mode'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['modes'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            logicalCh.id, 'mode', options=True)}),
                    initial=logicalCh.getMode().value,
                    required=False)
            else:
                self.fields['mode'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['modes'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            logicalCh.id, 'mode', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['mode'])
        except Exception:
            self.fields['mode'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['mode'])

        # pockelCellSetting
        try:
            if logicalCh.pockelCellSetting is not None:
                self.fields['pockelCellSetting'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    initial=logicalCh.pockelCellSetting,
                    label="Pockel cell",
                    required=False)
            else:
                self.fields['pockelCellSetting'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(logicalCh.id, 'name')}),
                    label="Pockel cell",
                    required=False)
            set_widget_attrs(self.fields['pockelCellSetting'])
        except Exception:
            self.fields['pockelCellSetting'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Pockel cell",
                required=False)
            set_widget_attrs(self.fields['pockelCellSetting'])

        self.fields.keyOrder = [
            'name', 'excitationWave', 'emissionWave', 'ndFilter',
            'pinHoleSize', 'fluor', 'illumination', 'contrastMethod', 'mode',
            'pockelCellSetting']


class MetadataDichroicForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataDichroicForm, self).__init__(*args, **kwargs)

        # Manufacturer
        try:
            if kwargs['initial']['dichroic'].manufacturer is not None:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].id,
                            'manufacturer')}),
                    initial=kwargs['initial']['dichroic'].manufacturer,
                    required=False)
            else:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].id,
                            'manufacturer')}),
                    required=False)
            set_widget_attrs(self.fields['manufacturer'])
        except Exception:
            self.fields['manufacturer'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['manufacturer'])

        # Model
        try:
            if kwargs['initial']['dichroic'].model is not None:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].id, 'model')}),
                    initial=kwargs['initial']['dichroic'].model,
                    required=False)
            else:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].id, 'model')}),
                    required=False)
            set_widget_attrs(self.fields['model'])
        except Exception:
            self.fields['model'] = forms.CharField(
                max_length=10, widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A", required=False)
            set_widget_attrs(self.fields['model'])

        # Serial number
        try:
            if kwargs['initial']['dichroic'].serialNumber is not None:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].serialNumber,
                            'serialNumber')}),
                    initial=kwargs['initial']['dichroic'].serialNumber,
                    label="Serial number",
                    required=False)
            else:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].serialNumber,
                            'serialNumber')}),
                    label="Serial number",
                    required=False)
            set_widget_attrs(self.fields['serialNumber'])
        except Exception:
            self.fields['serialNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Serial number",
                required=False)
            set_widget_attrs(self.fields['serialNumber'])

        # Lot number
        try:
            if kwargs['initial']['dichroic'].lotNumber is not None:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].lotNumber,
                            'lotNumber')}),
                    initial=kwargs['initial']['dichroic'].lotNumber,
                    label="Lot number",
                    required=False)
            else:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['dichroic'].lotNumber,
                            'lotNumber')}),
                    label="Lot number",
                    required=False)
            set_widget_attrs(self.fields['lotNumber'])
        except Exception:
            self.fields['lotNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Lot number",
                required=False)
            set_widget_attrs(self.fields['lotNumber'])

        self.fields.keyOrder = [
            'model', 'manufacturer', 'serialNumber', 'lotNumber']


class MetadataMicroscopeForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataMicroscopeForm, self).__init__(*args, **kwargs)

        # Model
        try:
            if kwargs['initial']['microscope'].model is not None:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id, 'model')}),
                    initial=kwargs['initial']['microscope'].model,
                    required=False)
            else:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id, 'model')}),
                    required=False)
            set_widget_attrs(self.fields['model'])
        except Exception:
            self.fields['model'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['model'])

        # Manufacturer
        try:
            if kwargs['initial']['microscope'].manufacturer is not None:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id,
                            'manufacturer')}),
                    initial=kwargs['initial']['microscope'].manufacturer,
                    required=False)
            else:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id,
                            'manufacturer')}),
                    required=False)
            set_widget_attrs(self.fields['manufacturer'])
        except Exception:
            self.fields['manufacturer'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['manufacturer'])

        # Serial number
        try:
            if kwargs['initial']['microscope'].serialNumber is not None:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id,
                            'lotNumber')}),
                    initial=kwargs['initial']['microscope'].serialNumber,
                    label="Serial number",
                    required=False)
            else:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id,
                            'lotNumber')}),
                    label="Serial number",
                    required=False)
            set_widget_attrs(self.fields['serialNumber'])
        except Exception:
            self.fields['serialNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Serial number",
                required=False)
            set_widget_attrs(self.fields['serialNumber'])

        # Lot number
        try:
            if kwargs['initial']['microscope'].lotNumber is not None:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id,
                            'lotNumber')}),
                    initial=kwargs['initial']['microscope'].lotNumber,
                    label="Lot number",
                    required=False)
            else:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id,
                            'lotNumber')}),
                    label="Serial number",
                    required=False)
            set_widget_attrs(self.fields['lotNumber'])
        except Exception:
            self.fields['lotNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Serial number",
                required=False)
            set_widget_attrs(self.fields['lotNumber'])

        # Type
        try:
            if (kwargs['initial']['microscope'].getMicroscopeType() is not
                    None):
                self.fields['type'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['microscopeTypes'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id, 'type',
                            options=True)}),
                    initial=kwargs['initial'][
                        'microscope'].getMicroscopeType().value,
                    required=False)
            else:
                self.fields['type'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['microscopeTypes'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['microscope'].id, 'type',
                            options=True)}),
                    required=False)
            set_widget_attrs(self.fields['type'])
        except Exception:
            self.fields['type'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['type'])

        self.fields.keyOrder = [
            'model', 'manufacturer', 'serialNumber', 'lotNumber', 'type']


class MetadataObjectiveForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataObjectiveForm, self).__init__(*args, **kwargs)

        # Model
        try:
            if kwargs['initial']['objective'].model is not None:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id, 'model')}),
                    initial=kwargs['initial']['objective'].model,
                    required=False)
            else:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id, 'model')}),
                    required=False)
            set_widget_attrs(self.fields['model'])
        except Exception:
            self.fields['model'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['model'])

        # Manufacturer
        try:
            if kwargs['initial']['objective'].manufacturer is not None:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'manufacturer')}),
                    initial=kwargs['initial']['objective'].manufacturer,
                    required=False)
            else:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'manufacturer')}),
                    required=False)
            set_widget_attrs(self.fields['manufacturer'])
        except Exception:
            self.fields['manufacturer'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['manufacturer'])

        # Serial Number
        try:
            if kwargs['initial']['objective'].serialNumber is not None:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'serialNumber')}),
                    initial=kwargs['initial']['objective'].serialNumber,
                    label="Serial number",
                    required=False)
            else:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'serialNumber')}),
                    label="Serial number",
                    required=False)
            set_widget_attrs(self.fields['serialNumber'])
        except Exception:
            self.fields['serialNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Serial number",
                required=False)
            set_widget_attrs(self.fields['serialNumber'])

        # Lot number
        try:
            if kwargs['initial']['objective'].lotNumber is not None:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].lotNumber,
                            'lotNumber')}),
                    initial=kwargs['initial']['objective'].lotNumber,
                    label="Lot number",
                    required=False)
            else:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial'][
                                'logicalchannel'].getObjective().lotNumber,
                            'lotNumber')}),
                    label="Lot number",
                    required=False)
            set_widget_attrs(self.fields['lotNumber'])
        except Exception:
            self.fields['lotNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Lot number",
                required=False)
            set_widget_attrs(self.fields['lotNumber'])

        # Nominal Magnification
        try:
            if (kwargs['initial']['objective'].nominalMagnification is not
                    None):
                self.fields['nominalMagnification'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'nominalMagnification')}),
                    initial=kwargs['initial'][
                        'objective'].nominalMagnification,
                    label="Nominal magnification",
                    required=False)
            else:
                self.fields['nominalMagnification'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'nominalMagnification')}),
                    label="Nominal magnification",
                    required=False)
            set_widget_attrs(self.fields['nominalMagnification'])
        except Exception:
            self.fields['nominalMagnification'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Nominal magnification",
                required=False)
            set_widget_attrs(self.fields['nominalMagnification'])

        # Calibrated Magnification
        try:
            if (kwargs['initial']['objective'].calibratedMagnification is not
                    None):
                self.fields['calibratedMagnification'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'calibratedMagnification')}),
                    initial=kwargs['initial'][
                        'objective'].calibratedMagnification,
                    label="Calibrated magnification",
                    required=False)
            else:
                self.fields['calibratedMagnification'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'calibratedMagnification')}),
                    label="Calibrated magnification",
                    required=False)
            set_widget_attrs(self.fields['calibratedMagnification'])
        except Exception:
            self.fields['calibratedMagnification'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Calibrated magnification",
                required=False)
            set_widget_attrs(self.fields['calibratedMagnification'])

        # Lens NA
        try:
            if kwargs['initial']['objective'].lensNA is not None:
                self.fields['lensNA'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id, 'lensNA')}),
                    initial=kwargs['initial']['objective'].lensNA,
                    label="Lens NA",
                    required=False)
            else:
                self.fields['lensNA'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id, 'lensNA')}),
                    required=False)
            set_widget_attrs(self.fields['lensNA'])
        except Exception:
            self.fields['lensNA'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Lens NA",
                required=False)
            set_widget_attrs(self.fields['lensNA'])

        # Immersion
        try:
            if kwargs['initial']['objective'].getImmersion() is not None:
                self.fields['immersion'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['immersions'],
                    empty_label=u"Not set", widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'immersion', options=True)}),
                    initial=kwargs['initial'][
                        'objective'].getImmersion().value,
                    required=False)
            else:
                self.fields['immersion'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['immersions'],
                    empty_label=u"Not set", widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'immersion', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['immersion'])
        except Exception:
            self.fields['immersion'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['immersion'])

        # Correction
        try:
            if kwargs['initial']['objective'].getCorrection() is not None:
                self.fields['correction'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['corrections'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'correction', options=True)}),
                    initial=kwargs['initial'][
                        'objective'].getCorrection().value,
                    required=False)
            else:
                self.fields['correction'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['corrections'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'correction', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['correction'])

        except Exception:
            self.fields['correction'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['correction'])

        # Working Distance
        try:
            if kwargs['initial']['objective'].workingDistance is not None:
                self.fields['workingDistance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'workingDistance')}),
                    initial=kwargs['initial'][
                        'objective'].workingDistance.getValue(),
                    label=("Working distance (%s)" % kwargs['initial'][
                           'objective'].workingDistance.getSymbol()),
                    required=False)
            else:
                self.fields['workingDistance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id,
                            'workingDistance')}),
                    label="Working distance",
                    required=False)
            set_widget_attrs(self.fields['workingDistance'])
        except Exception:
            self.fields['workingDistance'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Working distance",
                required=False)
            set_widget_attrs(self.fields['workingDistance'])

        # Iris
        try:
            if kwargs['initial']['objective'].getIris() is not None:
                self.fields['iris'] = forms.ChoiceField(
                    choices=self.BOOLEAN_CHOICES,  widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id, 'iris',
                            options=True)}),
                    initial=kwargs['initial']['objective'].getIris().value,
                    required=False)
            else:
                self.fields['iris'] = forms.ChoiceField(
                    choices=self.BOOLEAN_CHOICES,
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objective'].id, 'iris',
                            options=True)}),
                    required=False)
            set_widget_attrs(self.fields['iris'])
        except Exception:
            self.fields['iris'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['iris'])

        self.fields.keyOrder = [
            'model', 'manufacturer', 'serialNumber', 'lotNumber',
            'nominalMagnification', 'calibratedMagnification', 'lensNA',
            'immersion', 'correction', 'workingDistance', 'iris']


class MetadataObjectiveSettingsForm(MetadataObjectiveForm):

    BOOLEAN_CHOICES = (
        ('', '---------'),
        ('True', 'True'),
        ('False', 'False'),
    )

    def __init__(self, *args, **kwargs):
        super(MetadataObjectiveSettingsForm, self).__init__(*args, **kwargs)

        # Objective Settings

        # Correction Collar
        try:
            if (kwargs['initial']['objectiveSettings'].correctionCollar is not
                    None):
                self.fields['correctionCollar'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objectiveSettings'].id,
                            'correctionCollar')}),
                    initial=kwargs['initial'][
                        'objectiveSettings'].correctionCollar,
                    label="Correction collar",
                    required=False)
            else:
                self.fields['correctionCollar'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objectiveSettings'].id,
                            'correctionCollar')}),
                        label="Correction collar",
                        required=False)
            set_widget_attrs(self.fields['correctionCollar'])
        except Exception:
            self.fields['correctionCollar'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Correction collar",
                required=False)
            set_widget_attrs(self.fields['correctionCollar'])

        # Medium
        try:
            if kwargs['initial']['objectiveSettings'].getMedium() is not None:
                self.fields['medium'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['mediums'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objectiveSettings'].id,
                            'medium', options=True)}),
                    initial=kwargs['initial'][
                        'objectiveSettings'].getMedium().value,
                    required=False)
            else:
                self.fields['medium'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['mediums'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['objectiveSettings'].id,
                            'medium', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['medium'])
        except Exception:
            self.fields['medium'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['medium'])

        # Refractive Index
        try:
            if (kwargs['initial']['objectiveSettings'].refractiveIndex is not
                    None):
                self.fields['refractiveIndex'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objectiveSettings'].id,
                            'refractiveIndex')}),
                    initial=kwargs['initial'][
                        'objectiveSettings'].refractiveIndex,
                    label="Refractive index",
                    required=False)
            else:
                self.fields['refractiveIndex'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['objectiveSettings'].id,
                            'refractiveIndex')}),
                    label="Refractive index",
                    required=False)
            set_widget_attrs(self.fields['refractiveIndex'])
        except Exception:
            self.fields['refractiveIndex'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Refractive index",
                required=False)
            set_widget_attrs(self.fields['refractiveIndex'])

        self.fields.keyOrder = [
            'model', 'manufacturer', 'serialNumber', 'lotNumber',
            'nominalMagnification', 'calibratedMagnification', 'lensNA',
            'immersion', 'correction', 'workingDistance', 'iris',
            'correctionCollar',  'medium', 'refractiveIndex']


class MetadataFilterForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataFilterForm, self).__init__(*args, **kwargs)

        # Filter

        # Manufacturer
        try:
            if kwargs['initial']['filter'].manufacturer is not None:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'manufacturer')}),
                    initial=kwargs['initial']['filter'].manufacturer,
                    required=False)
            else:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'manufacturer')}),
                    required=False)
            set_widget_attrs(self.fields['manufacturer'])
        except Exception:
            self.fields['manufacturer'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['manufacturer'])

        # Model
        try:
            if kwargs['initial']['filter'].model is not None:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'model')}),
                    initial=kwargs['initial']['filter'].model,
                    required=False)
            else:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'model')}),
                    required=False)
            set_widget_attrs(self.fields['model'])
        except Exception:
            self.fields['model'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['model'])

        # Serial Number
        try:
            if kwargs['initial']['filter'].serialNumber is not None:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'serialNumber')}),
                    initial=kwargs['initial']['filter'].serialNumber,
                    label="Serial number",
                    required=False)
            else:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'serialNumber')}),
                    label="Serial number", required=False)
            set_widget_attrs(self.fields['serialNumber'])
        except Exception:
            self.fields['serialNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A", label="Serial number",
                required=False)
            set_widget_attrs(self.fields['serialNumber'])

        # Lot number
        try:
            if kwargs['initial']['filter'].lotNumber is not None:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'lotNumber')}),
                    initial=kwargs['initial']['filter'].lotNumber,
                    label="Lot number",
                    required=False)
            else:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'lotNumber')}),
                    label="Lot number",
                    required=False)
            set_widget_attrs(self.fields['lotNumber'])
        except Exception:
            self.fields['lotNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Lot number",
                required=False)
            set_widget_attrs(self.fields['lotNumber'])

        # Filter wheel
        try:
            if kwargs['initial']['filter'].filterWheel is not None:
                self.fields['filterWheel'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'filterWheel')}),
                    initial=kwargs['initial']['filter'].filterWheel,
                    label="Filter wheel",
                    required=False)
            else:
                self.fields['filterWheel'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'filterWheel')}),
                    label="Filter wheel",
                    required=False)
            set_widget_attrs(self.fields['filterWheel'])
        except Exception:
            self.fields['filterWheel'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Filter wheel",
                required=False)
            set_widget_attrs(self.fields['filterWheel'])

        # Type
        try:
            if kwargs['initial']['filter'].getFilterType() is not None:
                self.fields['type'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['types'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'type',
                            options=True)}),
                    initial=kwargs['initial']['filter'].getFilterType().value,
                    required=False)
            else:
                self.fields['type'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['types'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'type',
                            options=True)}),
                    required=False)
            set_widget_attrs(self.fields['type'])
        except Exception:
            self.fields['type'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['type'])

        # Cut in
        tr = kwargs['initial']['filter'].getTransmittanceRange()
        try:
            if tr is not None and tr.cutIn is not None:
                self.fields['cutIn'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'cutIn')}),
                    initial=kwargs['initial'][
                        'filter'].getTransmittanceRange().cutIn.getValue(),
                    label="Cut in (%s)" % tr.cutIn.getSymbol(),
                    required=False)
            else:
                self.fields['cutIn'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'cutIn')}),
                    label="Cut in",
                    required=False)
            set_widget_attrs(self.fields['cutIn'])
        except Exception:
            self.fields['cutIn'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Cut in",
                required=False)
            set_widget_attrs(self.fields['cutIn'])

        # Cut out
        try:
            if tr is not None and tr.cutOut is not None:
                self.fields['cutOut'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'cutOut')}),
                    initial=tr.cutOut.getValue(),
                    label="Cut out (%s)" % tr.cutOut.getSymbol(),
                    required=False)
            else:
                self.fields['cutOut'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'cutOut')}),
                    label="Cut out",
                    required=False)
            set_widget_attrs(self.fields['cutOut'])
        except Exception:
            self.fields['cutOut'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Cut out",
                required=False)
            set_widget_attrs(self.fields['cutOut'])

        # Cut in tolerance
        try:
            if tr is not None and tr.cutInTolerance is not None:
                self.fields['cutInTolerance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id,
                            'cutInTolerance')}),
                    initial=tr.cutInTolerance.getValue(),
                    label=("Cut in tolerance (%s)"
                           % tr.cutInTolerance.getSymbol()),
                    required=False)
            else:
                self.fields['cutInTolerance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id,
                            'cutInTolerance')}),
                    label="Cut in tolerance",
                    required=False)
            set_widget_attrs(self.fields['cutInTolerance'])
        except Exception:
            self.fields['cutInTolerance'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Cut in tolerance",
                required=False)
            set_widget_attrs(self.fields['cutInTolerance'])

        # Cut on tolerance
        try:
            if tr is not None and tr.cutOutTolerance is not None:
                self.fields['cutOutTolerance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'cutOut')}),
                    initial=tr.cutOutTolerance.getValue(),
                    label=("Cut out tolerance (%s)"
                           % tr.cutOutTolerance.getSymbol()),
                    required=False)
            else:
                self.fields['cutOutTolerance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id, 'cutOut')}),
                    label="Cut out tolerance",
                    required=False)
            set_widget_attrs(self.fields['cutOutTolerance'])
        except Exception:
            self.fields['cutOutTolerance'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Cut out tolerance",
                required=False)
            set_widget_attrs(self.fields['cutOutTolerance'])

        # Transmittance
        try:
            if kwargs['initial']['filter'].transmittanceRange is not None:
                self.fields['transmittance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id,
                            'transmittance')}),
                    initial=formatPercentFraction(
                        kwargs['initial'][
                            'filter'].getTransmittanceRange().transmittance),
                    label="Transmittance (%)",
                    required=False)
            else:
                self.fields['transmittance'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['filter'].id,
                            'transmittance')}),
                    required=False)
            set_widget_attrs(self.fields['transmittance'])
        except Exception:
            self.fields['transmittance'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['transmittance'])

        self.fields.keyOrder = [
            'model', 'manufacturer', 'serialNumber', 'lotNumber', 'type',
            'filterWheel', 'cutIn', 'cutOut', 'cutInTolerance',
            'cutOutTolerance', 'transmittance']


class MetadataDetectorForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataDetectorForm, self).__init__(*args, **kwargs)

        detSet = kwargs['initial']['detectorSettings']
        detector = kwargs['initial']['detector']

        # Manufacturer
        try:
            if detector is not None:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'manufacturer')}),
                    initial=detector.manufacturer,
                    required=False)
            else:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'manufacturer')}),
                    required=False)
            set_widget_attrs(self.fields['manufacturer'])
        except Exception:
            self.fields['manufacturer'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['manufacturer'])

        # Model
        try:
            if detector is not None:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detector.id, 'model')}),
                    initial=detector.model,
                    required=False)
            else:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detector.id, 'model')}),
                    required=False)
            set_widget_attrs(self.fields['model'])
        except Exception:
            self.fields['model'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['model'])

        # SN
        try:
            if detector is not None:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'serialNumber')}),
                    initial=detector.serialNumber,
                    required=False)
            else:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'serialNumber')}),
                    required=False)
            set_widget_attrs(self.fields['serialNumber'])
        except Exception:
            self.fields['serialNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['serialNumber'])

        # Lot number (NB. Untill OMERO model is updated in 4.3, this will
        # throw since lotNumber is not yet supported)
        try:
            if detector is not None:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detector.id, 'lotNumber')}),
                    initial=detector.lotNumber,
                    required=False)
            else:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detector.id, 'lotNumber')}),
                    required=False)
            set_widget_attrs(self.fields['lotNumber'])
        except Exception:
            self.fields['lotNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['lotNumber'])

        # Type
        try:
            if detector.getDetectorType() is not None:
                self.fields['type'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['types'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            detector.id, 'type', options=True)}),
                    initial=detector.getDetectorType().value,
                    required=False)
            else:
                self.fields['type'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['types'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            detector.id, 'type', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['type'])
        except Exception:
            self.fields['type'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['type'])

        # Gain
        try:
            if detSet is not None:
                self.fields['gain'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detSet.id, 'gain')}),
                    initial=detSet.gain,
                    required=False)
            elif detector is not None:
                self.fields['gain'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detector.id, 'gain')}),
                    initial=detector.gain,
                    required=False)
            else:
                self.fields['gain'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detSet.id, 'gain')}),
                    required=False)
            set_widget_attrs(self.fields['gain'])
        except Exception:
            self.fields['gain'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['gain'])

        # Voltage
        try:
            if detSet is not None and detSet.voltage is not None:
                self.fields['voltage'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detSet.id, 'voltage')}),
                    initial=detSet.voltage.getValue(),
                    label="Voltage (%s)" % detSet.voltage.getSymbol(),
                    required=False)
            elif detector is not None:
                self.fields['voltage'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detector.id, 'voltage')}),
                    initial=detector.voltage.getValue(),
                    label="Voltage (%s)" % detector.voltage.getSymbol(),
                    required=False)
            else:
                self.fields['voltage'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detSet.id, 'voltage')}),
                    required=False)
            set_widget_attrs(self.fields['voltage'])
        except Exception:
            self.fields['voltage'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['voltage'])

        # Offset
        try:
            if detSet is not None:
                self.fields['offsetValue'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detSet.id, 'offsetValue')}),
                    initial=detSet.offsetValue,
                    label="Offset",
                    required=False)
            elif detector is not None:
                self.fields['offsetValue'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'offsetValue')}),
                    initial=detector.offsetValue,
                    label="Offset",
                    required=False)
            else:
                self.fields['offsetValue'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'offsetValue')}),
                    label="Offset",
                    required=False)
            set_widget_attrs(self.fields['offsetValue'])
        except Exception:
            self.fields['offsetValue'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Offset",
                required=False)
            set_widget_attrs(self.fields['offsetValue'])

        # Zoom
        try:
            if detector is not None:
                self.fields['zoom'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'voltage')}),
                    initial=detector.zoom,
                    required=False)
            else:
                self.fields['zoom'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'voltage')}),
                    required=False)
            set_widget_attrs(self.fields['zoom'])
        except Exception:
            self.fields['zoom'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['zoom'])

        # Amplification gain
        try:
            if detector is not None:
                self.fields['amplificationGain'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'amplificationGain')}),
                    initial=detector.amplificationGain,
                    label="Amplification gain",
                    required=False)
            else:
                self.fields['amplificationGain'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            detector.id, 'amplificationGain')}),
                    label="Amplification gain",
                    required=False)
            set_widget_attrs(self.fields['amplificationGain'])
        except Exception:
            self.fields['amplificationGain'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Amplification gain",
                required=False)
            set_widget_attrs(self.fields['amplificationGain'])

        # Read out rate
        try:
            if detSet is not None and detSet.readOutRate is not None:
                self.fields['readOutRate'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detSet.id, 'readOutRate')}),
                    initial=detSet.readOutRate.getValue(),
                    label=("Read out rate (%s)"
                           % detSet.readOutRate.getSymbol()),
                    required=False)
            else:
                self.fields['readOutRate'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(detSet.id, 'readOutRate')}),
                    label="Read out rate",
                    required=False)
            set_widget_attrs(self.fields['readOutRate'])
        except Exception:
            self.fields['readOutRate'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Read out rate",
                required=False)
            set_widget_attrs(self.fields['readOutRate'])

        # Binning
        try:
            if detSet is not None:
                self.fields['binning'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['binnings'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            detSet.id, 'type', options=True)}),
                    initial=detSet.getBinning().value,
                    required=False)
            else:
                self.fields['binning'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['binnings'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            detSet.id, 'type', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['binning'])
        except Exception:
            self.fields['binning'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['binning'])

        self.fields.keyOrder = [
            'model', 'manufacturer', 'serialNumber', 'lotNumber', 'type',
            'gain', 'voltage', 'offsetValue', 'zoom', 'amplificationGain',
            'readOutRate', 'binning']


class MetadataLightSourceForm(forms.Form):

    BOOLEAN_CHOICES = (
        ('', '---------'),
        ('True', 'True'),
        ('False', 'False'),
    )

    def __init__(self, *args, **kwargs):
        super(MetadataLightSourceForm, self).__init__(*args, **kwargs)

        lightSource = kwargs['initial']['lightSource']
        lightSourceSettings = None
        if 'lightSourceSettings' in kwargs['initial']:
            lightSourceSettings = kwargs['initial']['lightSourceSettings']

        self.lightSourceType = lightSource.OMERO_CLASS

        # Manufacturer
        try:
            if lightSource.manufacturer is not None:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(lightSource.id, 'model')}),
                    initial=lightSource.manufacturer,
                    required=False)
            else:
                self.fields['manufacturer'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(lightSource.id, 'model')}),
                    required=False)
            set_widget_attrs(self.fields['manufacturer'])
        except Exception:
            self.fields['manufacturer'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['manufacturer'])

        # Model
        try:
            if lightSource.model is not None:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(lightSource.id, 'model')}),
                    initial=lightSource.model,
                    required=False)
            else:
                self.fields['model'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(lightSource.id, 'model')}),
                    required=False)
            set_widget_attrs(self.fields['model'])
        except Exception:
            self.fields['model'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['model'])

        # Serial Number
        try:
            if lightSource.serialNumber is not None:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'serialNumber')}),
                    initial=lightSource.serialNumber,
                    label="Serial number",
                    required=False)
            else:
                self.fields['serialNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'serialNumber')}),
                    label="Serial number",
                    required=False)
            set_widget_attrs(self.fields['serialNumber'])
        except Exception:
            self.fields['serialNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Serial number",
                required=False)
            set_widget_attrs(self.fields['serialNumber'])

        # Lot Number
        try:
            if lightSource.lotNumber is not None:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'lotNumber')}),
                    initial=lightSource.lotNumber,
                    label="Lot number",
                    required=False)
            else:
                self.fields['lotNumber'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'lotNumber')}),
                    label="Lot number",
                    required=False)
            set_widget_attrs(self.fields['lotNumber'])
        except Exception:
            self.fields['lotNumber'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Lot number",
                required=False)
            set_widget_attrs(self.fields['lotNumber'])

        # Power
        try:
            if lightSource.power is not None:
                self.fields['power'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(lightSource.id, 'power')}),
                    initial=lightSource.power.getValue(),
                    label="Power (%s)" % lightSource.power.getSymbol(),
                    required=False)
            else:
                self.fields['power'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(lightSource.id, 'power')}),
                    required=False)
            set_widget_attrs(self.fields['power'])
        except Exception:
            self.fields['power'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['power'])

        # Type
        try:
            if lightSource.getLightSourceType() is not None:
                self.fields['lstype'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['lstypes'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'type', options=True)}),
                    label="Type",
                    initial=lightSource.getLightSourceType().value,
                    required=False)
            else:
                self.fields['lstype'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['lstypes'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'type', options=True)}),
                    label="Type",
                    required=False)
            set_widget_attrs(self.fields['lstype'])
        except Exception:
            self.fields['lstype'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Type",
                required=False)
            set_widget_attrs(self.fields['lstype'])

        # Pump (laser only)
        try:
            # Will throw exception for non-Laser lightsources.
            pump = lightSource.getPump()
            pumpType = pump.OMERO_CLASS     # E.g. 'Arc'
            pumpModel = pump.getModel()
            pumpValue = "%s: %s" % (pumpType, pumpModel)
            self.fields['pump'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial=pumpValue,
                required=False)
        except Exception:
            # Not a Laser - don't show Pump
            self.fields['pump'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
        set_widget_attrs(self.fields['pump'])

        # Medium
        try:
            if lightSource.getLaserMedium() is not None:
                self.fields['lmedium'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['mediums'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'medium', options=True)}),
                    initial=lightSource.getLaserMedium().value,
                    label="Medium",
                    required=False)
            else:
                self.fields['lmedium'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['mediums'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'medium', options=True)}),
                    label="Medium",
                    required=False)
            set_widget_attrs(self.fields['lmedium'])
        except Exception:
            self.fields['lmedium'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Medium",
                required=False)
            set_widget_attrs(self.fields['lmedium'])

        # Wavelength
        try:
            if (lightSourceSettings is not None and
                    lightSourceSettings.wavelength is not None):
                self.fields['wavelength'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'wavelength')}),
                    initial=lightSourceSettings.wavelength.getValue(),
                    label=("Wavelength (%s)"
                           % lightSourceSettings.wavelength.getSymbol()),
                    required=False)
            elif lightSource.wavelength is not None:
                self.fields['wavelength'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'wavelength')}),
                    initial=lightSource.wavelength.getValue(),
                    label=("Wavelength (%s)"
                           % lightSource.wavelength.getSymbol()),
                    required=False)
            else:
                self.fields['wavelength'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'wavelength')}),
                    required=False)
            set_widget_attrs(self.fields['wavelength'])
        except Exception:
            self.fields['wavelength'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['wavelength'])

        # FrequencyMultiplication
        try:
            if lightSource.frequencyMultiplication is not None:
                self.fields['frequencyMultiplication'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'frequencyMultiplication')}),
                    initial=lightSource.frequencyMultiplication,
                    label="Frequency Multiplication",
                    required=False)
            else:
                self.fields['frequencyMultiplication'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'frequencyMultiplication')}),
                    label="Frequency Multiplication",
                    required=False)
            set_widget_attrs(self.fields['frequencyMultiplication'])
        except Exception:
            self.fields['frequencyMultiplication'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Frequency Multiplication",
                required=False)
            set_widget_attrs(self.fields['frequencyMultiplication'])

        # Tuneable
        try:
            if lightSource.tuneable is not None:
                self.fields['tuneable'] = forms.ChoiceField(
                    choices=self.BOOLEAN_CHOICES,
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'tuneable', options=True)}),
                    initial=lightSource.tuneable,
                    required=False)
            else:
                self.fields['tuneable'] = forms.ChoiceField(
                    choices=self.BOOLEAN_CHOICES,
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'tuneable', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['tuneable'])
        except Exception:
            self.fields['tuneable'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['tuneable'])

        # Pulse
        try:
            if lightSource.pulse is not None:
                self.fields['pulse'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['pulses'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'pulse', options=True)}),
                    initial=lightSource.pulse,
                    required=False)
            else:
                self.fields['pulse'] = MetadataModelChoiceField(
                    queryset=kwargs['initial']['pulses'],
                    empty_label=u"Not set",
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'pulse', options=True)}),
                    required=False)
            set_widget_attrs(self.fields['pulse'])
        except Exception:
            self.fields['pulse'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['pulse'])

        # Repetition Rate
        try:
            if lightSource.repetitionRate is not None:
                self.fields['repetitionRate'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'repetitionRate')}),
                    initial=lightSource.repetitionRate.getValue(),
                    label=("Repetition rate (%s)"
                           % lightSource.repetitionRate.getSymbol()),
                    required=False)
            else:
                self.fields['repetitionRate'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            lightSource.id, 'repetitionRate')}),
                    label="Repetition rate",
                    required=False)
            set_widget_attrs(self.fields['repetitionRate'])
        except Exception:
            self.fields['repetitionRate'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Repetition rate",
                required=False)
            set_widget_attrs(self.fields['repetitionRate'])

        # Pockel Cell
        try:
            if lightSource.pockelCell is not None:
                self.fields['pockelCell'] = forms.ChoiceField(
                    choices=self.BOOLEAN_CHOICES,
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'pockelCell', options=True)}),
                    initial=lightSource.pockelCell,
                    label="Pockel Cell",
                    required=False)
            else:
                self.fields['pockelCell'] = forms.ChoiceField(
                    choices=self.BOOLEAN_CHOICES,
                    widget=forms.Select(attrs={
                        'onchange': save_metadata(
                            lightSource.id, 'pockelCell', options=True)}),
                    label="Pockel Cell",
                    required=False)
            set_widget_attrs(self.fields['pockelCell'])
        except Exception:
            self.fields['pockelCell'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Pockel Cell",
                required=False)
            set_widget_attrs(self.fields['pockelCell'])

        # Attenuation
        if (lightSourceSettings is not None and
                lightSourceSettings.attenuation is not None):
            self.fields['attenuation'] = forms.CharField(
                max_length=100,
                widget=forms.TextInput(attrs={
                    'size': 25,
                    'onchange': save_metadata(
                        lightSourceSettings.id, 'attenuation')}),
                initial=formatPercentFraction(
                    lightSourceSettings.attenuation),
                label="Attenuation (%)",
                required=False)
        else:
            self.fields['attenuation'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
        set_widget_attrs(self.fields['attenuation'])

        self.fields.keyOrder = [
            'model', 'manufacturer', 'serialNumber', 'lotNumber', 'power',
            'lstype', 'pump', 'lmedium', 'wavelength',
            'frequencyMultiplication', 'tuneable', 'pulse', 'repetitionRate',
            'pockelCell', 'attenuation']


class MetadataEnvironmentForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataEnvironmentForm, self).__init__(*args, **kwargs)

        # Imaging environment

        imagingEnv = kwargs['initial']['image'].getImagingEnvironment()
        # Temperature
        try:
            if imagingEnv.temperature is not None:
                self.fields['temperature'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'temperature')}),
                    initial=imagingEnv.temperature.getValue(),
                    label=("Temperature (%s)"
                           % imagingEnv.temperature.getSymbol()),
                    required=False)
            else:
                self.fields['temperature'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'temperature')}),
                    required=False)
            set_widget_attrs(self.fields['temperature'])
        except Exception:
            self.fields['temperature'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['temperature'])

        # Air Pressure
        try:
            if imagingEnv.airPressure is not None:
                self.fields['airPressure'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'airPressure')}),
                    initial=imagingEnv.airPressure.getValue(),
                    label=("Air Pressure (%s)"
                           % imagingEnv.airPressure.getSymbol()),
                    required=False)
            else:
                self.fields['airPressure'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'airPressure')}),
                    label="Air Pressure",
                    required=False)
            set_widget_attrs(self.fields['airPressure'])
        except Exception:
            self.fields['airPressure'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                label="Air Pressure",
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['airPressure'])

        # Humidity
        try:
            if imagingEnv.humidity is not None:
                self.fields['humidity'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'humidity')}),
                    initial=imagingEnv.humidity,
                    required=False)
            else:
                self.fields['humidity'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'humidity')}),
                    required=False)
            set_widget_attrs(self.fields['humidity'])
        except Exception:
            self.fields['humidity'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                required=False)
            set_widget_attrs(self.fields['humidity'])

        # CO2 percent
        try:
            if imagingEnv.co2percent is not None:
                self.fields['co2percent'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'co2percent')}),
                    initial=imagingEnv.co2percent,
                    label="CO2 (%)",
                    required=False)
            else:
                self.fields['co2percent'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'co2percent')}),
                    label="CO2 (%)",
                    required=False)
            set_widget_attrs(self.fields['co2percent'])
        except Exception:
            self.fields['co2percent'] = forms.CharField(
                max_length=10,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="CO2 (%)",
                required=False)
            set_widget_attrs(self.fields['co2percent'])

        self.fields.keyOrder = [
            'airPressure', 'co2percent', 'humidity', 'temperature']


class MetadataStageLabelForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(MetadataStageLabelForm, self).__init__(*args, **kwargs)

        # Stage label

        # Position x
        try:
            if kwargs['initial']['image'].getStageLabel() is not None:
                self.fields['positionx'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'positionx')}),
                    initial=kwargs['initial'][
                        'image'].getStageLabel().positionx,
                    label="Position X",
                    required=False)
            else:
                self.fields['positionx'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'positionx')}),
                    label="Position X",
                    required=False)
            set_widget_attrs(self.fields['positionx'])
        except Exception:
            self.fields['positionx'] = forms.CharField(
                max_length=100,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Position X",
                required=False)
            set_widget_attrs(self.fields['positionx'])

        # Position y
        try:
            if kwargs['initial']['image'].getStageLabel() is not None:
                self.fields['positiony'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'positiony')}),
                    initial=kwargs['initial'][
                        'image'].getStageLabel().positiony,
                    label="Position Y",
                    required=False)
            else:
                self.fields['positiony'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'positiony')}),
                    label="Position Y",
                    required=False)
            set_widget_attrs(self.fields['positiony'])
        except Exception:
            self.fields['positiony'] = forms.CharField(
                max_length=100,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A",
                label="Position Y",
                required=False)
            set_widget_attrs(self.fields['positionx'])

        # Position z
        try:
            if kwargs['initial']['image'].getStageLabel() is not None:
                self.fields['positionz'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'positionz')}),
                    initial=kwargs['initial'][
                        'image'].getStageLabel().positionz,
                    label="Position Z",
                    required=False)
            else:
                self.fields['positionz'] = forms.CharField(
                    max_length=100,
                    widget=forms.TextInput(attrs={
                        'size': 25,
                        'onchange': save_metadata(
                            kwargs['initial']['image'].id, 'positionz')}),
                    label="Position Z",
                    required=False)
            set_widget_attrs(self.fields['positionz'])
        except Exception:
            self.fields['positionz'] = forms.CharField(
                max_length=100,
                widget=forms.TextInput(attrs={'size': 25}),
                initial="N/A", label="Position Z", required=False)
            set_widget_attrs(self.fields['positionz'])

        self.fields.keyOrder = ['positionx', 'positiony', 'positionz']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
   Library for Web integration tests
"""
from __future__ import print_function

from future import standard_library
import json
import warnings

from django.test import Client
from django.test.client import MULTIPART_CONTENT
from django.core.urlresolvers import reverse

from omero.testlib import ITest

standard_library.install_aliases() # noqa
from urllib.parse import urlencode


class IWebTest(ITest):
    """
    Abstract class derived from ITest which implements helpers for creating
    Django clients using django.test
    """

    @classmethod
    def setup_class(cls):
        """Returns a logged in Django test client."""
        super(IWebTest, cls).setup_class()
        cls.django_clients = []
        ome_name = cls.ctx.userName
        cls.django_client = cls.new_django_client(ome_name, ome_name)
        rootpass = cls.root.ic.getProperties().getProperty('omero.rootpass')
        cls.django_root_client = cls.new_django_client("root", rootpass)

    @classmethod
    def teardown_class(cls):
        logout_url = reverse('weblogout')
        for client in cls.django_clients:
            data = {'csrfmiddlewaretoken': client.cookies['csrftoken'].value}
            response = client.post(logout_url, data=data)
            assert response.status_code == 302
        super(IWebTest, cls).teardown_class()

    @classmethod
    def new_django_client(cls, name, password):
        django_client = Client(enforce_csrf_checks=True)
        login_url = reverse('weblogin')

        response = django_client.get(login_url)
        assert response.status_code == 200
        csrf_token = django_client.cookies['csrftoken'].value

        data = {
            'server': 1,
            'username': name,
            'password': password,
            'csrfmiddlewaretoken': csrf_token
        }
        response = django_client.post(login_url, data)
        assert response.status_code == 302
        cls.django_clients.append(django_client)
        return django_client

    @classmethod
    def new_django_client_from_session_id(cls, session_id):
        django_client = Client(enforce_csrf_checks=True)
        index_url = reverse('webindex')

        data = {
            'server': 1,
            'bsession': session_id,
        }
        response = django_client.get(index_url, data)
        assert response.status_code == 200
        cls.django_clients.append(django_client)
        return django_client

    def import_image_with_metadata(self, client=None):
        """
        Imports fake image replacing tinyTest. Wavelength not supported yet
        """
        warnings.warn(
            "This method is deprecated as of OMERO 5.5.0. Use"
            " omero.test_lib.import_fake_file",
            DeprecationWarning)
        images = self.import_fake_file(
            client=client, pixelType="int16", sizeX=20, sizeY=20, sizeZ=5,
            sizeT=6)
        return images[0]


# Helpers
def _response(django_client, request_url, method, data=None, status_code=200,
              content_type=MULTIPART_CONTENT, **extra):
    response = getattr(django_client, method)(request_url,
                                              data=data,
                                              content_type=content_type,
                                              **extra)
    assert response.status_code == status_code, response
    return response


def csrf_response(django_client, request_url, method, data=None,
                  status_code=200, content_type=MULTIPART_CONTENT,
                  test_csrf_required=True):
    """
    Helper for testing post/put/delete with and without CSRF.

    :param django_client:   Django test Client
    :param request_url:     The url to request
    :parma method:          Http method, e.g. 'post'
    :param data:            A dict of data to include as json content
    :param status_code:     Verify that the response has this status
    :param content_type:    Content type for request
    :param test_csrf_required:  If True (default) check that request fails
                                when CSRF token is not added

    """
    # First check that this would fail with 403 without CSRF token
    if test_csrf_required:
        _response(django_client, request_url, method=method, data=data,
                  status_code=403, content_type=content_type)

    # Should work as expected with CSRF token
    csrf_token = django_client.cookies['csrftoken'].value
    extra = {'HTTP_X_CSRFTOKEN': csrf_token}
    return _response(django_client, request_url, method=method, data=data,
                     status_code=status_code, content_type=content_type,
                     **extra)


# POST
def post(django_client, request_url, data=None, status_code=200,
         content_type=MULTIPART_CONTENT):
    """
    Performs a POST request, and returns the response.

    :param django_client:   Django test Client
    :param request_url:     The url to request
    :param data:            A dict of data to include as json content
    :param status_code:     Verify that the response has this status
    :param content_type:

    """
    return csrf_response(django_client, request_url, "post", data=data,
                         status_code=status_code, content_type=content_type)


def post_json(django_client, request_url, data=None, status_code=200):
    """
    Performs a POST request, and returns the JSON response as a dict.

    :param django_client:   Django test Client
    :param request_url:     The url to request
    :param data:            A dict of data to include as json content
    :param status_code:     Verify that the response has this status

    """
    rsp = post(django_client, request_url, json.dumps(data),
               status_code=status_code, content_type='application/json')
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


def _post_response(django_client, request_url, data, status_code=403,
                   content_type=MULTIPART_CONTENT, **extra):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use post_json",
        DeprecationWarning)
    return _response(django_client, request_url, method='post', data=data,
                     status_code=status_code, content_type=content_type,
                     **extra)


def _post_response_json(django_client, request_url, data, status_code=403,
                        content_type=MULTIPART_CONTENT, **extra):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use post_json",
        DeprecationWarning)
    rsp = _response(django_client, request_url, method='post', data=data,
                    status_code=status_code, content_type=content_type,
                    **extra)
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


def _csrf_post_response(django_client, request_url, data, status_code=200,
                        content_type=MULTIPART_CONTENT):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use post_json",
        DeprecationWarning)
    csrf_token = django_client.cookies['csrftoken'].value
    extra = {'HTTP_X_CSRFTOKEN': csrf_token}
    return _post_response(django_client, request_url, data=data,
                          status_code=status_code, content_type=content_type,
                          **extra)


def _csrf_post_response_json(django_client, request_url,
                             query_string, status_code=200):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use post_json",
        DeprecationWarning)
    rsp = _csrf_post_response(django_client, request_url,
                              query_string, status_code)
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


# POST json encoded as a string
def _csrf_post_json(django_client, request_url, data,
                    status_code=200, content_type='application/json'):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use post_json",
        DeprecationWarning)
    csrf_token = django_client.cookies['csrftoken'].value
    extra = {'HTTP_X_CSRFTOKEN': csrf_token}
    rsp = django_client.post(request_url, json.dumps(data),
                             status_code=status_code,
                             content_type=content_type,
                             **extra)
    print(rsp)
    assert rsp.status_code == status_code
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


def put_json(django_client, request_url, data=None, status_code=200):
    """
    Performs a PUT request, and returns the JSON response as a dict.

    :param django_client:   Django test Client
    :param request_url:     The url to request
    :param data:            A dict of data to include as json content
    :param status_code:     Verify that the response has this status

    """
    rsp = csrf_response(django_client, request_url, 'put', json.dumps(data),
                        status_code=status_code,
                        content_type='application/json')
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


# PUT json encoded as a string
def _csrf_put_json(django_client, request_url, data,
                   status_code=200, content_type='application/json'):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use put_json",
        DeprecationWarning)
    csrf_token = django_client.cookies['csrftoken'].value
    extra = {'HTTP_X_CSRFTOKEN': csrf_token}
    rsp = django_client.put(request_url, json.dumps(data),
                            status_code=status_code, content_type=content_type,
                            **extra)
    print(rsp)
    assert rsp.status_code == status_code
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


# DELETE
def _delete_response(django_client, request_url, data, status_code=403,
                     content_type=MULTIPART_CONTENT, **extra):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use delete_json",
        DeprecationWarning)
    return _response(django_client, request_url, method='delete', data=data,
                     status_code=status_code, content_type=content_type,
                     **extra)


def delete_json(django_client, request_url, data=None, status_code=200):
    """
    Performs a DELETE request, and returns the JSON response as a dict.

    :param django_client:   Django test Client
    :param request_url:     The url to request
    :param data:            A dict of data to include as json content
    :param status_code:     Verify that the response has this status

    """
    rsp = csrf_response(django_client, request_url, 'delete', json.dumps(data),
                        status_code=status_code,
                        content_type='application/json')
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


def _csrf_delete_response(django_client, request_url, data, status_code=200,
                          content_type=MULTIPART_CONTENT):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use delete_json",
        DeprecationWarning)
    csrf_token = django_client.cookies['csrftoken'].value
    extra = {'HTTP_X_CSRFTOKEN': csrf_token}
    return _delete_response(django_client, request_url, data=data,
                            status_code=status_code, content_type=content_type,
                            **extra)


def _csrf_delete_response_json(django_client, request_url,
                               data, status_code=200):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use delete_json",
        DeprecationWarning)
    rsp = _csrf_delete_response(django_client, request_url, data, status_code)
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


# GET
def get(django_client, request_url, data=None, status_code=200, csrf=False):
    """
    Performs a GET request and returns the response.

    :param django_client:   Django test Client
    :param request_url:     The url to request
    :param data:            A dictionary of data, used to build a query string
    :param status_code:     Verify that the response has this status
    :param csrf:            If true, add csrf token to query string
    """
    if csrf:
        if data is None:
            data = {}
        else:
            # avoid mutating dict we're passed
            data = data.copy()
        csrf_token = django_client.cookies['csrftoken'].value
        data['csrfmiddlewaretoken'] = csrf_token
    if data is not None:
        query_string = urlencode(list(data.items()), doseq=True)
        request_url = '%s?%s' % (request_url, query_string)
    return _response(django_client, request_url, 'get',
                     status_code=status_code)


def get_json(django_client, request_url, data=None, status_code=200,
             csrf=False):
    """
    Performs a GET request and returns the JSON response as a dict.

    :param django_client:   Django test Client
    :param request_url:     The url to request
    :param data:            A dictionary of data, used to build a query string
    :param status_code:     Verify that the response has this status
    :param csrf:            If true, add csrf token to query string

    """
    rsp = get(django_client, request_url, data, status_code, csrf)
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)


def _get_response(django_client, request_url, query_string, status_code=405):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use get",
        DeprecationWarning)
    query_string = urlencode(list(query_string.items()), doseq=True)
    response = django_client.get('%s?%s' % (request_url, query_string))
    assert response.status_code == status_code
    return response


def _csrf_get_response(django_client, request_url, query_string,
                       status_code=200):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use get",
        DeprecationWarning)
    csrf_token = django_client.cookies['csrftoken'].value
    query_string['csrfmiddlewaretoken'] = csrf_token
    return _get_response(django_client, request_url, query_string,
                         status_code)


def _get_response_json(django_client, request_url,
                       query_string, status_code=200):
    warnings.warn(
        "This method is deprecated as of OMERO 5.4.0. Use get_json",
        DeprecationWarning)
    rsp = _get_response(django_client, request_url, query_string, status_code)
    assert rsp.get('Content-Type') == 'application/json'
    return json.loads(rsp.content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2008-2018 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#

import logging

try:
    from collections import OrderedDict  # Python 2.7+ only
except Exception:
    pass

from django import forms
from django.forms.widgets import Textarea

from omeroweb.connector import Server

from omeroweb.custom_forms import NonASCIIForm

from .custom_forms import ServerModelChoiceField, GroupModelChoiceField
from .custom_forms import GroupModelMultipleChoiceField, OmeNameField
from .custom_forms import ExperimenterModelMultipleChoiceField, MultiEmailField

logger = logging.getLogger(__name__)

#################################################################
# Non-model Form


class LoginForm(NonASCIIForm):

    def __init__(self, *args, **kwargs):
        super(LoginForm, self).__init__(*args, **kwargs)
        self.fields['server'] = ServerModelChoiceField(
            Server, empty_label=None)

        self.fields.keyOrder = ['server', 'username', 'password']

    username = forms.CharField(
        max_length=50, widget=forms.TextInput(attrs={
            'size': 22, 'autofocus': 'autofocus'}))
    password = forms.CharField(
        max_length=50,
        widget=forms.PasswordInput(attrs={'size': 22, 'autocomplete': 'off'}))

    def clean_username(self):
        if (self.cleaned_data['username'] == 'guest'):
            raise forms.ValidationError("Guest account is not supported.")
        return self.cleaned_data['username']


class ForgottonPasswordForm(NonASCIIForm):

    def __init__(self, *args, **kwargs):
        super(ForgottonPasswordForm, self).__init__(*args, **kwargs)
        self.fields['server'] = ServerModelChoiceField(Server,
                                                       empty_label=None)
        f = forms.CharField(max_length=50,
                            widget=forms.TextInput(
                                attrs={'size': 28, 'autocomplete': 'off'}))
        self.fields['username'] = f
        self.fields['email'] = forms.EmailField(
            widget=forms.TextInput(attrs={'size': 28, 'autocomplete': 'off'}))


ROLE_CHOICES = (
    ('user', 'User'),
    ('administrator', 'Administrator'),
    ('restricted_administrator', 'Administrator with restricted privileges')
)


class ExperimenterForm(NonASCIIForm):

    def __init__(self, name_check=False, email_check=False,
                 experimenter_is_me_or_system=False,
                 experimenter_me=False,
                 can_modify_user=True,
                 user_privileges=[],
                 experimenter_root=False,
                 *args, **kwargs):
        super(ExperimenterForm, self).__init__(*args, **kwargs)
        self.name_check = name_check
        self.email_check = email_check
        self.user_privileges = user_privileges

        try:
            self.fields['other_groups'] = GroupModelMultipleChoiceField(
                queryset=kwargs['initial']['groups'],
                initial=kwargs['initial']['other_groups'], required=False,
                label="Groups")
        except Exception:
            self.fields['other_groups'] = GroupModelMultipleChoiceField(
                queryset=kwargs['initial']['groups'], required=False,
                label="Groups")

        try:
            self.fields['default_group'] = GroupModelChoiceField(
                queryset=kwargs['initial']['my_groups'],
                initial=kwargs['initial']['default_group'],
                empty_label=u"", required=False)
        except Exception:
            try:
                self.fields['default_group'] = GroupModelChoiceField(
                    queryset=kwargs['initial']['my_groups'],
                    empty_label=u"", required=False)
            except Exception:
                self.fields['default_group'] = GroupModelChoiceField(
                    queryset=list(), empty_label=u"", required=False)

        # 'Role' is disabled if experimenter is 'admin' or self,
        # so required=False to avoid validation error.
        # If current user is restricted Admin, can't create full Admin
        restricted_admin = "ReadSession" not in self.user_privileges
        self.fields['role'] = forms.ChoiceField(
            choices=ROLE_CHOICES,
            widget=forms.RadioSelect(
                {'disabled': restricted_admin or experimenter_root}),
            required=False,
            initial='user')

        if ('with_password' in kwargs['initial'] and
                kwargs['initial']['with_password']):
            self.fields['password'] = forms.CharField(
                max_length=50,
                widget=forms.PasswordInput(attrs={'size': 30,
                                                  'autocomplete': 'off'}))
            self.fields['confirmation'] = forms.CharField(
                max_length=50,
                widget=forms.PasswordInput(attrs={'size': 30,
                                                  'autocomplete': 'off'}))

            fields_key_order = [
                'omename', 'password', 'confirmation', 'first_name',
                'middle_name', 'last_name', 'email', 'institution',
                'role', 'active', 'other_groups', 'default_group']
        else:
            fields_key_order = [
                'omename', 'first_name', 'middle_name', 'last_name',
                'email', 'institution', 'role', 'active',
                'other_groups', 'default_group']

        ordered_fields = [(k, self.fields[k]) for k in fields_key_order]

        roles = [('Sudo', 'Sudo'),
                 # combine WriteFile/ManagedRepo/Owned roles into 'Write'
                 ('Write', 'Write Data'),
                 # combine DeleteFile/ManagedRepo/Owned roles into 'Delete'
                 ('Delete', 'Delete Data'),
                 ('Chgrp', 'Chgrp'),
                 ('Chown', 'Chown'),
                 ('ModifyGroup', 'Create and Edit Groups'),
                 ('ModifyUser', 'Create and Edit Users'),
                 ('ModifyGroupMembership', 'Add Users to Groups'),
                 ('Script', 'Upload Scripts')]
        for role in roles:
            # If current user is light-admin, ignore privileges they don't have
            # So they can't add/remove these from experimenter
            # We don't disable them - (not in form data and will be removed)
            ordered_fields.append(
                (role[0], forms.BooleanField(
                    required=False,
                    label=role[1],
                    widget=forms.CheckboxInput(
                        attrs={'class': 'privilege',
                               'disabled': role[0] not in user_privileges})
                ))
            )

        # Django 1.8: Form.fields uses OrderedDict from the collections module.
        self.fields = OrderedDict(ordered_fields)

        if experimenter_me or experimenter_root:
            self.fields['omename'].widget.attrs['readonly'] = True
            name = "yourself"
            if experimenter_root:
                name = "'root' user"
            self.fields['omename'].widget.attrs['title'] = \
                "You can't edit Username of %s" % name
            self.fields['role'].widget.attrs['disabled'] = True
            self.fields['active'].widget.attrs['disabled'] = True
            self.fields['active'].widget.attrs['title'] = \
                "You cannot disable %s" % name

        # If we can't modify user, ALL fields are disabled
        if not can_modify_user:
            for field in self.fields.values():
                field.widget.attrs['disabled'] = True

    omename = OmeNameField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}),
        label="Username")
    first_name = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}))
    middle_name = forms.CharField(max_length=250, widget=forms.TextInput(
        attrs={'size': 30, 'autocomplete': 'off'}), required=False)
    last_name = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}))
    email = forms.EmailField(
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}),
        required=False)
    institution = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}),
        required=False)

    active = forms.BooleanField(required=False)

    def clean_confirmation(self):
        if self.cleaned_data.get('password'):
            if len(self.cleaned_data.get('password')) < 3:
                raise forms.ValidationError(
                    'Password must be at least 3 characters long.')
        if (self.cleaned_data.get('password') or
                self.cleaned_data.get('confirmation')):
            if (self.cleaned_data.get('password') !=
                    self.cleaned_data.get('confirmation')):
                raise forms.ValidationError('Passwords do not match')
            else:
                return self.cleaned_data.get('password')

    def clean_omename(self):
        if self.name_check:
            raise forms.ValidationError('This username already exists.')
        return self.cleaned_data.get('omename')

    def clean_email(self):
        if self.email_check:
            raise forms.ValidationError('This email already exists.')
        return self.cleaned_data.get('email')

    def clean_default_group(self):
        if (self.cleaned_data.get('default_group') is None or
                len(self.cleaned_data.get('default_group')) <= 0):
            raise forms.ValidationError('No default group selected.')
        else:
            return self.cleaned_data.get('default_group')

    def clean_other_groups(self):
        if (self.cleaned_data.get('other_groups') is None or
                len(self.cleaned_data.get('other_groups')) <= 0):
            raise forms.ValidationError(
                'User must be a member of at least one group.')
        else:
            return self.cleaned_data.get('other_groups')


PERMISSION_CHOICES = (
    ('0', 'Private'),
    ('1', 'Read-Only'),
    ('2', 'Read-Annotate'),
    ('3', 'Read-Write'),
)


class GroupForm(NonASCIIForm):

    def __init__(self, name_check=False, group_is_system=False,
                 can_modify_group=True, can_add_member=True, *args, **kwargs):
        super(GroupForm, self).__init__(*args, **kwargs)

        self.name_check = name_check
        if can_modify_group:

            self.fields['name'] = forms.CharField(
                max_length=100,
                widget=forms.TextInput(attrs={'size': 25,
                                              'autocomplete': 'off'}))
            self.fields['description'] = forms.CharField(
                max_length=250, required=False,
                widget=forms.TextInput(attrs={'size': 25,
                                              'autocomplete': 'off'}))
        if can_add_member:
            try:
                if kwargs['initial']['owners']:
                    pass
                self.fields['owners'] = ExperimenterModelMultipleChoiceField(
                    queryset=kwargs['initial']['experimenters'],
                    initial=kwargs['initial']['owners'], required=False)
            except Exception:
                self.fields['owners'] = ExperimenterModelMultipleChoiceField(
                    queryset=kwargs['initial']['experimenters'],
                    required=False)

            try:
                if kwargs['initial']['members']:
                    pass
                self.fields['members'] = ExperimenterModelMultipleChoiceField(
                    queryset=kwargs['initial']['experimenters'],
                    initial=kwargs['initial']['members'], required=False)
            except Exception:
                self.fields['members'] = ExperimenterModelMultipleChoiceField(
                    queryset=kwargs['initial']['experimenters'],
                    required=False)

        if can_modify_group:
            self.fields['permissions'] = forms.ChoiceField(
                choices=PERMISSION_CHOICES, widget=forms.RadioSelect(),
                required=True, label="Permissions")

        if group_is_system:
            self.fields['name'].widget.attrs['readonly'] = True
            self.fields['name'].widget.attrs['title'] = \
                "Changing of system group name would be un-doable"

        self.fields.keyOrder = [
            'name', 'description', 'owners', 'members', 'permissions']

        # If we can't modify group, disable fields
        if not can_modify_group:
            for name, field in self.fields.items():
                if name not in ('owners', 'members'):
                    field.widget.attrs['disabled'] = True

    def clean_name(self):
        if self.name_check:
            raise forms.ValidationError('This name already exists.')
        return self.cleaned_data.get('name')


class GroupOwnerForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(GroupOwnerForm, self).__init__(*args, **kwargs)

        try:
            if kwargs['initial']['owners']:
                pass
            self.fields['owners'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'],
                initial=kwargs['initial']['owners'], required=False)
        except Exception:
            self.fields['owners'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'], required=False)

        try:
            if kwargs['initial']['members']:
                pass
            self.fields['members'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'],
                initial=kwargs['initial']['members'], required=False)
        except Exception:
            self.fields['members'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'], required=False)

        self.fields.keyOrder = ['owners', 'members', 'permissions']

    permissions = forms.ChoiceField(
        choices=PERMISSION_CHOICES, widget=forms.RadioSelect(), required=True,
        label="Permissions")


class MyAccountForm(NonASCIIForm):

    def __init__(self, email_check=False, *args, **kwargs):
        super(MyAccountForm, self).__init__(*args, **kwargs)
        self.email_check = email_check
        try:
            if kwargs['initial']['default_group']:
                pass
            self.fields['default_group'] = GroupModelChoiceField(
                queryset=kwargs['initial']['groups'],
                initial=kwargs['initial']['default_group'],
                empty_label=None)
        except Exception:
            self.fields['default_group'] = GroupModelChoiceField(
                queryset=kwargs['initial']['groups'],
                empty_label=None)
        self.fields.keyOrder = [
            'omename', 'first_name', 'middle_name', 'last_name', 'email',
            'institution', 'default_group']

    omename = forms.CharField(
        max_length=50,
        widget=forms.TextInput(attrs={'onfocus': 'this.blur()', 'size': 30,
                                      'autocomplete': 'off'}),
        label="Username")
    first_name = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}))
    middle_name = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}),
        required=False)
    last_name = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}))
    email = forms.EmailField(
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}),
        required=False)
    institution = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30, 'autocomplete': 'off'}),
        required=False)

    def clean_email(self):
        if self.email_check:
            raise forms.ValidationError('This email already exists.')
        return self.cleaned_data.get('email')


class ContainedExperimentersForm(NonASCIIForm):

    def __init__(self, *args, **kwargs):
        super(ContainedExperimentersForm, self).__init__(*args, **kwargs)

        try:
            if kwargs['initial']['members']:
                pass
            self.fields['members'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'],
                initial=kwargs['initial']['members'],
                required=False)
        except Exception:
            self.fields['members'] = ExperimenterModelMultipleChoiceField(
                queryset=kwargs['initial']['experimenters'],
                required=False)

        self.fields.keyOrder = ['members']


class UploadPhotoForm(forms.Form):

    photo = forms.FileField(required=False)

    def clean_photo(self):
        if self.cleaned_data.get('photo') is None:
            raise forms.ValidationError(
                'No image selected. Supported image formats (file extensions'
                ' allowed): jpeg, jpg, gif, png. The maximum image size'
                ' allowed is 200KB.')
        if not self.cleaned_data.get(
                'photo').content_type.startswith("image"):
            raise forms.ValidationError(
                'Supported image formats (file extensions allowed):'
                ' jpeg, jpg, gif, png.')
        if self.cleaned_data.get('photo').size > 204800:
            raise forms.ValidationError(
                'The maximum image size allowed is 200KB.')
        return self.cleaned_data.get('photo')


class ChangePassword(NonASCIIForm):

    old_password = forms.CharField(
        max_length=50,
        widget=forms.PasswordInput(attrs={'size': 30, 'autocomplete': 'off'}),
        label="Current password")
    password = forms.CharField(
        max_length=50,
        widget=forms.PasswordInput(attrs={'size': 30, 'autocomplete': 'off'}),
        label="New password")
    confirmation = forms.CharField(
        max_length=50,
        widget=forms.PasswordInput(attrs={'size': 30, 'autocomplete': 'off'}),
        label="Confirm password")

    def clean_confirmation(self):
        if (self.cleaned_data.get('password') or
                self.cleaned_data.get('confirmation')):
            if len(self.cleaned_data.get('password')) < 3:
                raise forms.ValidationError('Password must be at least 3'
                                            ' characters long.')
            if (self.cleaned_data.get('password') !=
                    self.cleaned_data.get('confirmation')):
                raise forms.ValidationError('Passwords do not match')
            else:
                return self.cleaned_data.get('password')


class EnumerationEntry(NonASCIIForm):

    new_entry = forms.CharField(
        max_length=250,
        widget=forms.TextInput(attrs={'size': 30}))


class EnumerationEntries(NonASCIIForm):

    def __init__(self, entries, *args, **kwargs):
        super(EnumerationEntries, self).__init__(*args, **kwargs)
        for i, e in enumerate(entries):
            try:
                if kwargs['initial']['entries']:
                    self.fields[str(e.id)] = forms.CharField(
                        max_length=250,
                        initial=e.value,
                        widget=forms.TextInput(attrs={'size': 30}),
                        label=i+1)
                else:
                    self.fields[str(e.id)] = forms.CharField(
                        max_length=250,
                        widget=forms.TextInput(attrs={'size': 30}),
                        label=i+1)
            except Exception:
                self.fields[str(e.id)] = forms.CharField(
                    max_length=250,
                    widget=forms.TextInput(attrs={'size': 30}),
                    label=i+1)

        self.fields.keyOrder = [str(k) for k in self.fields.keys()]


class EmailForm(forms.Form):
    """
    Form to gather recipients, subject and message for sending email
    announcements
    """

    error_css_class = 'field-error'
    required_css_class = 'field-required'

    # Define these as None just so I can order them
    everyone = forms.BooleanField(required=False, label='All Users')

    experimenters = forms.TypedMultipleChoiceField(
        required=False,
        coerce=int,
        label='Users'
    )
    groups = forms.TypedMultipleChoiceField(
        required=False,
        coerce=int
    )

    # TODO CC isn't really CC. Maybe change label or change functionality
    cc = MultiEmailField(required=False)
    subject = forms.CharField(max_length=100, required=True)
    message = forms.CharField(widget=Textarea, required=True)

    # Include/Exclude inactive users
    inactive = forms.BooleanField(label='Include inactive users',
                                  required=False)

    def __init__(self, experimenters, groups, conn, request, *args, **kwargs):
        super(EmailForm, self).__init__(*args, **kwargs)
        # Process Experimenters/Groups into choices (lists of tuples)
        self.fields['experimenters'].choices = [
            (experimenter.id, experimenter.firstName +
             ' ' + experimenter.lastName + ' (' + experimenter.omeName + ')' +
             (' - Inactive' if not experimenter.isActive() else ''))
            for experimenter in experimenters]

        self.fields['groups'].choices = [
            (group.id, group.name) for group in groups]

        self.conn = conn
        self.request = request

    def clean(self):
        cleaned_data = super(EmailForm, self).clean()
        everyone = cleaned_data.get("everyone")
        experimenters = cleaned_data.get("experimenters")
        groups = cleaned_data.get("groups")
        cc = cleaned_data.get("cc")

        # If nobody addressed, throw an error
        if not cc and not everyone and not experimenters and not groups:
            raise forms.ValidationError("At least one addressee must be "
                                        "specified in one or more of 'all',"
                                        " 'user', 'group' or 'cc'")
        return cleaned_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#
# Copyright (c) 2008-2016 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#

from django.conf import settings
from django.conf.urls import url

from omeroweb.webclient import views
from omeroweb.webgateway import views as webgateway
from omeroweb.webclient.webclient_gateway import defaultThumbnail
from django.core.urlresolvers import get_callable

viewer_view = get_callable(settings.VIEWER_VIEW)

urlpatterns = [

    # Home page is the main 'Data' page
    url(r'^$', views.load_template, {'menu': 'userdata'}, name="webindex"),

    # render main template
    url(r'^(?P<menu>((?i)userdata|public|history|search|help|usertags))/$',
        views.load_template,
        name="load_template"),
    url(r'^userdata/$',
        views.load_template, {'menu': 'userdata'},
        name="userdata"),
    url(r'^history/$',
        views.load_template, {'menu': 'history'},
        name="history"),

    url(r'^login/$', views.WebclientLoginView.as_view(), name="weblogin"),
    url(r'^logout/$', views.logout, name="weblogout"),
    url(r'^active_group/$',
        views.change_active_group,
        name="change_active_group"),

    # The content of group/users drop-down menu
    url(r'^group_user_content/$',
        views.group_user_content,
        name="group_user_content"),

    # update, display activities, E.g. delete queues, scripts etc.
    url(r'^activities/', views.activities, name="activities"),
    url(r'^activities_json/',
        views.activities,
        {'template': 'json'},
        name="activities_json"),
    url(r'^activities_update/(?:(?P<action>clean)/)?$',
        views.activities_update,
        name="activities_update"),

    # loading data
    url(r'^load_plate/(?:(?P<o1_type>'
        r'((?i)plate|acquisition))/)'
        r'?(?:(?P<o1_id>[0-9]+)/)?$',
        views.load_plate,
        name="load_plate"),

    # chgrp. Load potential target groups, then load target P/D within chosen
    # group
    url(r'^load_chgrp_groups/$',
        views.load_chgrp_groups,
        name="load_chgrp_groups"),  # Query E.g. ?Image=1,2&Dataset=3
    url(r'^load_chgrp_target/(?P<group_id>[0-9]+)/'
        r'(?P<target_type>((?i)project|dataset|screen))/$',
        views.load_chgrp_target,
        name="load_chgrp_target"),

    # load history
    url(r'^load_calendar/(?:(\d{4})/(\d{1,2})/)?$', views.load_calendar,
        name="load_calendar"),
    url(r'^load_history/(?:(\d{4})/(\d{1,2})/(\d{1,2})/)?$',
        views.load_history, name="load_history"),

    # load search
    url(r'^load_searching/(?:(?P<form>((?i)form))/)?$', views.load_searching,
        name="load_searching"),

    # metadata
    url(r'^metadata_details/(?:(?P<c_type>[a-zA-Z]+)/'
        r'(?P<c_id>[0-9]+)/)?(?:(?P<share_id>[0-9]+)/)?$',
        views.load_metadata_details,
        name="load_metadata_details"),
    url(r'^metadata_acquisition/(?P<c_type>[a-zA-Z]+)/'
        r'(?P<c_id>[0-9]+)/(?:(?P<share_id>[0-9]+)/)?$',
        views.load_metadata_acquisition,
        name="load_metadata_acquisition"),
    url(r'^metadata_preview/(?P<c_type>((?i)image|well))/'
        r'(?P<c_id>[0-9]+)/(?:(?P<share_id>[0-9]+)/)?$',
        views.load_metadata_preview,
        name="load_metadata_preview"),
    url(r'^metadata_hierarchy/(?P<c_type>[a-zA-Z]+)/'
        r'(?P<c_id>[0-9]+)/(?:(?P<share_id>[0-9]+)/)?$',
        views.load_metadata_hierarchy,
        name="load_metadata_hierarchy"),

    url(r'^get_thumbnails/(?:(?P<share_id>[0-9]+)/)?$',
        webgateway.get_thumbnails_json,
        name="get_thumbnails_json"),
    url(r'^get_thumbnail/(?P<iid>[0-9]+)/'
        r'(?:(?P<share_id>[0-9]+)/)?$',
        webgateway.get_thumbnail_json,
        {'_defcb': defaultThumbnail},
        name="get_thumbnail_json"),
    url(r'^render_thumbnail/(?P<iid>[0-9]+)/'
        r'(?:(?P<share_id>[0-9]+)/)?$',
        webgateway.render_thumbnail,
        {'_defcb': defaultThumbnail},
        name="render_thumbnail"),
    url(r'^render_thumbnail/size/(?P<w>[0-9]+)/'
        r'(?P<iid>[0-9]+)/(?:(?P<share_id>[0-9]+)/)?$',
        webgateway.render_thumbnail,
        {'_defcb': defaultThumbnail},
        name="render_thumbnail_resize"),
    url(r'^edit_channel_names/(?P<imageId>[0-9]+)/$',
        views.edit_channel_names,
        name="edit_channel_names"),

    # image webgateway extention
    url(r'^(?:(?P<share_id>[0-9]+)/)?render_image_region/'
        r'(?P<iid>[0-9]+)/(?P<z>[0-9]+)/(?P<t>[0-9]+)/$',
        webgateway.render_image_region,
        name="web_render_image_region"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?render_birds_eye_view/'
        r'(?P<iid>[^/]+)/(?:(?P<size>[^/]+)/)?$',
        webgateway.render_birds_eye_view,
        name="web_render_birds_eye_view"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?render_image/(?P<iid>[^/]+)/'
        r'(?:(?P<z>[^/]+)/)?(?:(?P<t>[^/]+)/)?$',
        webgateway.render_image,
        name="web_render_image"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?render_image_download/'
        r'(?P<iid>[^/]+)/(?:(?P<z>[^/]+)/)?(?:(?P<t>[^/]+)/)?$',
        webgateway.render_image,
        {'download': True},
        name="web_render_image_download"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?img_detail/(?P<iid>[0-9]+)/$',
        viewer_view,
        name="web_image_viewer"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?imgData/(?P<iid>[0-9]+)/$',
        webgateway.imageData_json,
        name="web_imageData_json"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?render_row_plot/(?P<iid>[^/]+)/'
        r'(?P<z>[^/]+)/(?P<t>[^/]+)/(?P<y>[^/]+)/(?:(?P<w>[^/]+)/)?$',
        webgateway.render_row_plot,
        name="web_render_row_plot"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?render_col_plot/(?P<iid>[^/]+)/'
        r'(?P<z>[^/]+)/(?P<t>[^/]+)/(?P<x>[^/]+)/(?:(?P<w>[^/]+)/)?$',
        webgateway.render_col_plot,
        name="web_render_col_plot"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?render_split_channel/'
        r'(?P<iid>[^/]+)/(?P<z>[^/]+)/(?P<t>[^/]+)/$',
        webgateway.render_split_channel,
        name="web_render_split_channel"),
    url(r'^saveImgRDef/(?P<iid>[^/]+)/$',
        webgateway.save_image_rdef_json,
        name="web_save_image_rdef_json"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?getImgRDef/$',
        webgateway.get_image_rdef_json,
        name="web_get_image_rdef_json"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?copyImgRDef/$',
        webgateway.copy_image_rdef_json,
        name="copy_image_rdef_json"),
    url(r'^(?:(?P<share_id>[0-9]+)/)?luts/$',
        webgateway.listLuts_json,
        name="web_list_luts"),


    # Fileset query (for delete or chgrp dialogs) obj-types and ids in REQUEST
    # data
    url(r'^fileset_check/(?P<action>((?i)delete|chgrp))/$',
        views.fileset_check,
        name="fileset_check"),

    # chgrp dry run - 'group_id', obj-types and ids in POST data.
    # E.g. Dataset=1,2,3 & Fileset=4. Multiple datatypes in one chgrp.
    url(r'^chgrpDryRun/$', views.chgrpDryRun, name="chgrpDryRun"),

    # Popup for downloading original archived files for images
    url(r'^download_placeholder/$', views.download_placeholder,
        name="download_placeholder"),

    # chgrp - 'group_id', obj-types and ids in POST data
    url(r'^chgrp/$', views.chgrp, name="chgrp"),

    # annotations
    url(r'^action/(?P<action>[a-zA-Z]+)/(?:(?P<o_type>[a-zA-Z]+)/)'
        r'?(?:(?P<o_id>[0-9]+)/)?$',
        views.manage_action_containers,
        name="manage_action_containers"),
    url(r'^batch_annotate/$', views.batch_annotate, name="batch_annotate"),
    url(r'^annotate_tags/$', views.annotate_tags, name="annotate_tags"),
    url(r'^marshal_tagging_form_data/$', views.marshal_tagging_form_data,
        name="marshal_tagging_form_data"),
    url(r'^annotate_rating/$',
        views.annotate_rating,
        name="annotate_rating"),
    url(r'^annotate_comment/$',
        views.annotate_comment,
        name="annotate_comment"),
    url(r'^annotate_file/$', views.annotate_file, name="annotate_file"),
    url(r'^annotate_map/$', views.annotate_map, name="annotate_map"),
    url(r'^annotation/(?P<annId>[0-9]+)/$',
        views.download_annotation,
        name="download_annotation"),
    url(r'^load_original_metadata/(?P<imageId>[0-9]+)/'
        r'(?:(?P<share_id>[0-9]+)/)?$',
        views.load_original_metadata,
        name="load_original_metadata"),
    url(r'^download_orig_metadata/(?P<imageId>[0-9]+)/$',
        views.download_orig_metadata,
        name="download_orig_metadata"),
    url(r'^omero_table/(?P<file_id>[0-9]+)/(?:(?P<mtype>((?i)json|csv))/)?$',
        views.omero_table,
        name="omero_table"),

    url(r'^avatar/(?P<oid>[0-9]+)/$', views.avatar, name="avatar"),


    # scripting service urls
    url(r'^list_scripts/$',
        views.list_scripts,
        name="list_scripts"),  # returns html list of scripts - click to run
    url(r'^script_ui/(?P<scriptId>[0-9]+)/$',
        views.script_ui,
        name='script_ui'),  # shows a form for running a script
    url(r'^script_run/(?P<scriptId>[0-9]+)/$',
        views.script_run,
        name='script_run'),  # runs the script - parameters in POST
    url(r'^script_upload/$', views.script_upload, name='script_upload'),
    url(r'^get_original_file/(?:(?P<fileId>[0-9]+)/)?$',
        views.get_original_file,
        name="get_original_file"),  # for stderr, stdout etc
    url(r'^download_original_file/(?:(?P<fileId>[0-9]+)/)?$',
        views.get_original_file,
        {'download': True},
        name="download_original_file"),  # for stderr, stdout etc
    url(r'^figure_script/(?P<scriptName>'
        r'((?i)SplitView|Thumbnail|MakeMovie))/$',
        views.figure_script,
        name='figure_script'),  # shows a form for running a script

    # ome_tiff_script: generate OME-TIFF and attach to image (use script
    # service). Must be POST
    url(r'^ome_tiff_script/(?P<imageId>[0-9]+)/$',
        views.ome_tiff_script,
        name='ome_tiff_script'),
    url(r'^ome_tiff_info/(?P<imageId>[0-9]+)/$',
        views.ome_tiff_info,
        name='ome_tiff_info'),

    # ping OMERO server to keep session alive
    url(r'^keepalive_ping/$', views.keepalive_ping, name="keepalive_ping"),

    # Load data, but with JSON.
    # url(r'^api/$', None, name='api'),
    url(r'^api/groups/$', views.api_group_list,
        name='api_groups'),

    url(r'^api/experimenters/$', views.api_experimenter_list,
        name='api_experimenters'),
    url(r'^api/experimenters/(?P<experimenter_id>-?\d+)/$',
        views.api_experimenter_detail, name='api_experimenter'),

    # Generic container list. This is necessary as an experimenter may have
    # datasets/etc which do not belong to any project
    url(r'^api/containers/$', views.api_container_list, name='api_containers'),

    # url(r'^api/projects/$', views.api_project_list, name='api_projects'),
    # url(r'^api/projects/(?P<pk>[0-9]+)/$', views.api_project_detail),

    url(r'^api/datasets/$', views.api_dataset_list, name='api_datasets'),
    # url(r'^api/datasets/(?P<pk>[0-9]+)/$', views.api_dataset_detail),

    url(r'^api/images/$', views.api_image_list, name='api_images'),

    # special case: share_id not allowed in query string since we
    # just want to allow share connection for this url ONLY.
    url(r'^api/share_images/(?P<share_id>[0-9]+)/$', views.api_image_list,
        name='api_share_images'),

    url(r'^api/plates/$', views.api_plate_list, name='api_plates'),
    # url(r'^api/plates/(?P<pk>[0-9]+)/$', views.api_plate_detail),

    url(r'^api/plate_acquisitions/$', views.api_plate_acquisition_list,
        name='api_plate_acquisitions'),
    # url(r'^api/plate_acquisitions/(?P<pk>[0-9]+)/$',
    #     views.api_plate_acquisitions_detail),

    # POST to create link, DELETE to remove.
    # links in request.body json, e.g. {"dataset":{"10":{"image":[1,2,3]}}}
    url(r'^api/links/$', views.api_links, name='api_links'),

    # url(r'^api/tags/$', views.api_tag_list, name='api_tags'),
    # url(r'^api/tags/(?P<pk>[0-9]+)/$', views.api_tag_detail),

    # Retrieve paths to an object
    url(r'^api/paths_to_object/$', views.api_paths_to_object,
        name='api_paths_to_object'),

    # Get parents of 1 or more objects. ?image=1,2&dataset=3
    url(r'^api/parent_links/$', views.api_parent_links,
        name='api_parent_links'),

    url(r'^api/tags/$', views.api_tags_and_tagged_list,
        name='api_tags_and_tagged'),

    url(r'^api/annotations/$', views.api_annotations,
        name='api_annotations'),

    url(r'^api/shares/$', views.api_share_list, name='api_shares'),
]
